﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Windows.Input;

namespace BlobStore
{
    // Simple DelegateCommand class; use a more official implementation for "real" projects
    public class DelegateCommand : ICommand
    {
        public Func<object, bool> CanExecuteDelegate { get; set; }
        public Action<object> ExecuteDelegate { get; set; }

        public bool CanExecute(object parameter)
        {
            if (null != CanExecuteDelegate)
            {
                return CanExecuteDelegate(parameter);
            }
            else
            {
                return true;
            }
        }

        public void Execute(object parameter)
        {
            if (null != ExecuteDelegate)
            {
                ExecuteDelegate(parameter);
            }
        }

        public void InvokeCanExecuteChanged()
        {
            var handler = CanExecuteChanged;
            if (null != handler)
            {
                handler.Invoke(this, EventArgs.Empty);
            }
        }

        public event EventHandler CanExecuteChanged;
    }
}
