﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System.Windows;
using System.Windows.Controls;

namespace BlobStore
{
    public partial class Progress : UserControl
    {
        private ProgressView _progressView = new ProgressView();

        public Progress()
        {
            InitializeComponent();

            Root.DataContext = _progressView;
        }

        public long ProgressTotal
        {
            get { return (long)GetValue(ProgressTotalProperty); }
            set { SetValue(ProgressTotalProperty, value); }
        }
        public static readonly DependencyProperty ProgressTotalProperty =
            DependencyProperty.Register("ProgressTotal", typeof(long), typeof(Progress), new PropertyMetadata((long)-1, OnProgressTotalChanged));
        private static void OnProgressTotalChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            ((Progress)o).OnProgressTotalChanged((long)e.OldValue, (long)e.NewValue);
        }
        protected virtual void OnProgressTotalChanged(long oldValue, long newValue)
        {
            if (-1 == newValue)
            {
                // Hide control
                ShowControl(false);
            }
            else if (0 == newValue)
            {
                // Show indeterminate
                ShowControl(true);
                _progressView.IsIndeterminate = true;
            }
            else
            {
                // Show progress
                ShowControl(true);
                _progressView.IsIndeterminate = false;
                _progressView.Progress = ProgressCurrent / ProgressTotal;
            }
        }

        public long ProgressCurrent
        {
            get { return (long)GetValue(ProgressCurrentProperty); }
            set { SetValue(ProgressCurrentProperty, value); }
        }
        public static readonly DependencyProperty ProgressCurrentProperty =
            DependencyProperty.Register("ProgressCurrent", typeof(long), typeof(Progress), new PropertyMetadata((long)0, OnProgressCurrentChanged));
        private static void OnProgressCurrentChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            ((Progress)o).OnProgressCurrentChanged((long)e.OldValue, (long)e.NewValue);
        }
        protected virtual void OnProgressCurrentChanged(long oldValue, long newValue)
        {
            _progressView.Progress = (double)ProgressCurrent / (double)ProgressTotal;
        }

        public string Message
        {
            get { return (string)GetValue(MessageProperty); }
            set { SetValue(MessageProperty, value); }
        }
        public static readonly DependencyProperty MessageProperty =
            DependencyProperty.Register("Message", typeof(string), typeof(Progress), new PropertyMetadata("", OnMessageChanged));
        private static void OnMessageChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            ((Progress)o).OnMessageChanged((string)e.OldValue, (string)e.NewValue);
        }
        protected virtual void OnMessageChanged(string oldValue, string newValue)
        {
            _progressView.Message = newValue;
        }

        public void ShowControl(bool show)
        {
            VisualStateManager.GoToState(this, show ? "Show" : "Hide", true);
        }
    }

    public class ProgressView : NotifyPropertyChangedBase
    {
        public bool IsIndeterminate
        {
            get { return _IsIndeterminate; }
            set
            {
                _IsIndeterminate = value;
                InvokePropertyChanged("IsIndeterminate");
            }
        }
        private bool _IsIndeterminate;

        public double Progress
        {
            get { return _progress; }
            set
            {
                _progress = value;
                InvokePropertyChanged("Progress");
            }
        }
        private double _progress;

        public string Message
        {
            get { return _message; }
            set
            {
                _message = value;
                InvokePropertyChanged("Message");
            }
        }
        private string _message;
    }

#if DEBUG
    public class DesignTimeProgressView : ProgressView
    {
        public DesignTimeProgressView()
        {
            IsIndeterminate = false;
            Progress = 0.6;
            Message = "Status message...";
        }
    }
#endif
}
